@@ -1,5 +1,5 @@
 /* Low level interface to ptrace, for GDB when running under Unix.
-   Copyright (C) 1986-2017 Free Software Foundation, Inc.
+   Copyright (C) 1986-2018 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -24,7 +24,7 @@
 #include "terminal.h"
 #include "target.h"
 #include "gdbthread.h"
-#include "observer.h"
+#include "observable.h"
 #include <signal.h>
 #include <fcntl.h>
 #include "gdb_select.h"
@@ -46,7 +46,7 @@
 
 static void pass_signal (int);
 
-static void child_terminal_ours_1 (int);
+static void child_terminal_ours_1 (target_terminal_state);
 
 /* Record terminal status separately for debugger and inferior.  */
 
@@ -54,8 +54,8 @@ static struct serial *stdin_serial;
 
 /* Terminal related info we need to keep track of.  Each inferior
    holds an instance of this structure --- we save it whenever the
-   corresponding inferior stops, and restore it to the foreground
-   inferior when it resumes.  */
+   corresponding inferior stops, and restore it to the terminal when
+   the inferior is resumed in the foreground.  */
 struct terminal_info
 {
   /* The name of the tty (from the `tty' command) that we gave to the
@@ -63,11 +63,23 @@ struct terminal_info
   char *run_terminal;
 
   /* TTY state.  We save it whenever the inferior stops, and restore
-     it when it resumes.  */
+     it when it resumes in the foreground.  */
   serial_ttystate ttystate;
 
 #ifdef HAVE_TERMIOS_H
-  /* Process group.  Saved and restored just like ttystate.  */
+  /* The terminal's foreground process group.  Saved whenever the
+     inferior stops.  This is the pgrp displayed by "info terminal".
+     Note that this may be not the inferior's actual process group,
+     since each inferior that we spawn has its own process group, and
+     only one can be in the foreground at a time.  When the inferior
+     resumes, if we can determine the inferior's actual pgrp, then we
+     make that the foreground pgrp instead of what was saved here.
+     While it's a bit arbitrary which inferior's pgrp ends up in the
+     foreground when we resume several inferiors, this at least makes
+     'resume inf1+inf2' + 'stop all' + 'resume inf2' end up with
+     inf2's pgrp in the foreground instead of inf1's (which would be
+     problematic since it would be left stopped: Ctrl-C wouldn't work,
+     for example).  */
   pid_t process_group;
 #endif
 
@@ -120,17 +132,6 @@ private:
 #endif
 };
 
-#ifdef HAVE_TERMIOS_H
-
-/* Return the process group of the current inferior.  */
-
-pid_t
-inferior_process_group (void)
-{
-  return get_inflow_inferior_data (current_inferior ())->process_group;
-}
-#endif
-
 /* While the inferior is running, we want SIGINT and SIGQUIT to go to the
    inferior only.  If we have job control, that takes care of it.  If not,
    we save our handlers in these two variables and set SIGINT and SIGQUIT
@@ -146,11 +147,17 @@ static sighandler_t sigquit_ours;
    fork_inferior, while forking a new child.  */
 static const char *inferior_thisrun_terminal;
 
-/* Nonzero if our terminal settings are in effect.  Zero if the
-   inferior's settings are in effect.  Ignored if !gdb_has_a_terminal
-   ().  */
+/* Track who owns GDB's terminal (is it GDB or some inferior?).  While
+   target_terminal::is_ours() etc. tracks the core's intention and is
+   independent of the target backend, this tracks the actual state of
+   GDB's own tty.  So for example,
 
-int terminal_is_ours;
+     (target_terminal::is_inferior () && gdb_tty_state == terminal_is_ours)
+
+   is true when the (native) inferior is not sharing a terminal with
+   GDB (e.g., because we attached to an inferior that is running on a
+   different terminal).  */
+static target_terminal_state gdb_tty_state = target_terminal_state::is_ours;
 
 /* See terminal.h.  */
 
@@ -196,29 +203,21 @@ gdb_has_a_terminal (void)
 void
 child_terminal_init (struct target_ops *self)
 {
-  struct inferior *inf = current_inferior ();
-  struct terminal_info *tinfo = get_inflow_inferior_data (inf);
+  if (!gdb_has_a_terminal ())
+    return;
+
+  inferior *inf = current_inferior ();
+  terminal_info *tinfo = get_inflow_inferior_data (inf);
 
 #ifdef HAVE_TERMIOS_H
-  /* Store the process group even without a terminal as it is used not
-     only to reset the tty foreground process group, but also to
-     interrupt the inferior.  A child we spawn should be a process
-     group leader (PGID==PID) at this point, though that may not be
-     true if we're attaching to an existing process.  */
+  /* A child we spawn should be a process group leader (PGID==PID) at
+     this point, though that may not be true if we're attaching to an
+     existing process.  */
   tinfo->process_group = inf->pid;
 #endif
 
-  if (gdb_has_a_terminal ())
-    {
-      xfree (tinfo->ttystate);
-      tinfo->ttystate = serial_copy_tty_state (stdin_serial,
-					       initial_gdb_ttystate);
-
-      /* Make sure that next time we call terminal_inferior (which will be
-         before the program runs, as it needs to be), we install the new
-         process group.  */
-      terminal_is_ours = 1;
-    }
+  xfree (tinfo->ttystate);
+  tinfo->ttystate = serial_copy_tty_state (stdin_serial, initial_gdb_ttystate);
 }
 
 /* Save the terminal settings again.  This is necessary for the TUI
@@ -235,31 +234,137 @@ gdb_save_tty_state (void)
     }
 }
 
-/* Put the inferior's terminal settings into effect.
-   This is preparation for starting or resuming the inferior.
+/* Try to determine whether TTY is GDB's input terminal.  Returns
+   TRIBOOL_UNKNOWN if we can't tell.  */
 
-   N.B. Targets that want to use this with async support must build that
-   support on top of this (e.g., the caller still needs to remove stdin
-   from the event loop).  E.g., see linux_nat_terminal_inferior.  */
+static tribool
+is_gdb_terminal (const char *tty)
+{
+  struct stat gdb_tty;
+  struct stat other_tty;
+  int res;
+
+  res = stat (tty, &other_tty);
+  if (res == -1)
+    return TRIBOOL_UNKNOWN;
+
+  res = fstat (STDIN_FILENO, &gdb_tty);
+  if (res == -1)
+    return TRIBOOL_UNKNOWN;
+
+  return ((gdb_tty.st_dev == other_tty.st_dev
+	   && gdb_tty.st_ino == other_tty.st_ino)
+	  ? TRIBOOL_TRUE
+	  : TRIBOOL_FALSE);
+}
+
+/* Helper for sharing_input_terminal.  Try to determine whether
+   inferior INF is using the same TTY for input as GDB is.  Returns
+   TRIBOOL_UNKNOWN if we can't tell.  */
+
+static tribool
+sharing_input_terminal_1 (inferior *inf)
+{
+  /* Using host-dependent code here is fine, because the
+     child_terminal_foo functions are meant to be used by child/native
+     targets.  */
+#if defined (__linux__) || defined (__sun__)
+  char buf[100];
+
+  xsnprintf (buf, sizeof (buf), "/proc/%d/fd/0", inf->pid);
+  return is_gdb_terminal (buf);
+#else
+  return TRIBOOL_UNKNOWN;
+#endif
+}
+
+/* Return true if the inferior is using the same TTY for input as GDB
+   is.  If this is true, then we save/restore terminal flags/state.
+
+   This is necessary because if inf->attach_flag is set, we don't
+   offhand know whether we are sharing a terminal with the inferior or
+   not.  Attaching a process without a terminal is one case where we
+   do not; attaching a process which we ran from the same shell as GDB
+   via `&' is one case where we do.
+
+   If we can't determine, we assume the TTY is being shared.  This
+   works OK if you're only debugging one inferior.  However, if you're
+   debugging more than one inferior, and e.g., one is spawned by GDB
+   with "run" (sharing terminal with GDB), and another is attached to
+   (and running on a different terminal, as is most common), then it
+   matters, because we can only restore the terminal settings of one
+   of the inferiors, and in that scenario, we want to restore the
+   settings of the "run"'ed inferior.
+
+   Note, this is not the same as determining whether GDB and the
+   inferior are in the same session / connected to the same
+   controlling tty.  An inferior (fork child) may call setsid,
+   disconnecting itself from the ctty, while still leaving
+   stdin/stdout/stderr associated with the original terminal.  If
+   we're debugging that process, we should also save/restore terminal
+   settings.  */
+
+static bool
+sharing_input_terminal (inferior *inf)
+{
+  terminal_info *tinfo = get_inflow_inferior_data (inf);
+
+  tribool res = sharing_input_terminal_1 (inf);
+
+  if (res == TRIBOOL_UNKNOWN)
+    {
+      /* As fallback, if we can't determine by stat'ing the inferior's
+	 tty directly (because it's not supported on this host) and
+	 the child was spawned, check whether run_terminal is our tty.
+	 This isn't ideal, since this is checking the child's
+	 controlling terminal, not the input terminal (which may have
+	 been redirected), but is still better than nothing.  A false
+	 positive ("set inferior-tty" points to our terminal, but I/O
+	 was redirected) is much more likely than a false negative
+	 ("set inferior-tty" points to some other terminal, and then
+	 output was redirected to our terminal), and with a false
+	 positive we just end up trying to save/restore terminal
+	 settings when we didn't need to or we actually can't.  */
+      if (tinfo->run_terminal != NULL)
+	res = is_gdb_terminal (tinfo->run_terminal);
+
+      /* If we still can't determine, assume yes.  */
+      if (res == TRIBOOL_UNKNOWN)
+	return true;
+    }
+
+  return res == TRIBOOL_TRUE;
+}
+
+/* Put the inferior's terminal settings into effect.  This is
+   preparation for starting or resuming the inferior.  */
 
 void
 child_terminal_inferior (struct target_ops *self)
 {
-  struct inferior *inf;
-  struct terminal_info *tinfo;
-
-  if (!terminal_is_ours)
+  /* If we resume more than one inferior in the foreground on GDB's
+     terminal, then the first inferior's terminal settings "win".
+     Note that every child process is put in its own process group, so
+     the first process that ends up resumed ends up determining which
+     process group the kernel forwards Ctrl-C/Ctrl-Z (SIGINT/SIGTTOU)
+     to.  */
+  if (gdb_tty_state == target_terminal_state::is_inferior)
     return;
 
-  inf = current_inferior ();
-  tinfo = get_inflow_inferior_data (inf);
+  inferior *inf = current_inferior ();
+  terminal_info *tinfo = get_inflow_inferior_data (inf);
 
   if (gdb_has_a_terminal ()
       && tinfo->ttystate != NULL
-      && tinfo->run_terminal == NULL)
+      && sharing_input_terminal (inf))
     {
       int result;
 
+      /* Ignore SIGTTOU since it will happen when we try to set the
+	 terminal's state (if gdb_tty_state is currently
+	 ours_for_output).  */
+      scoped_ignore_sigttou ignore_sigttou;
+
 #ifdef F_GETFL
       result = fcntl (0, F_SETFL, tinfo->tflags);
       OOPSY ("fcntl F_SETFL");
@@ -276,28 +381,40 @@ child_terminal_inferior (struct target_ops *self)
 #endif
 	}
 
-      /* If attach_flag is set, we don't know whether we are sharing a
-         terminal with the inferior or not.  (attaching a process
-         without a terminal is one case where we do not; attaching a
-         process which we ran from the same shell as GDB via `&' is
-         one case where we do, I think (but perhaps this is not
-         `sharing' in the sense that we need to save and restore tty
-         state)).  I don't know if there is any way to tell whether we
-         are sharing a terminal.  So what we do is to go through all
-         the saving and restoring of the tty state, but ignore errors
-         setting the process group, which will happen if we are not
-         sharing a terminal).  */
-
       if (job_control)
 	{
 #ifdef HAVE_TERMIOS_H
+	  /* If we can't tell the inferior's actual process group,
+	     then restore whatever was the foreground pgrp the last
+	     time the inferior was running.  See also comments
+	     describing terminal_state::process_group.  */
+#ifdef HAVE_GETPGID
+	  result = tcsetpgrp (0, getpgid (inf->pid));
+#else
 	  result = tcsetpgrp (0, tinfo->process_group);
-	  if (!inf->attach_flag)
-	    OOPSY ("tcsetpgrp");
+#endif
+	  if (result == -1)
+	    {
+#if 0
+	      /* This fails if either GDB has no controlling terminal,
+		 e.g., running under 'setsid(1)', or if the inferior
+		 is not attached to GDB's controlling terminal.  E.g.,
+		 if it called setsid to create a new session or used
+		 the TIOCNOTTY ioctl, or simply if we've attached to a
+		 process running on another terminal and we couldn't
+		 tell whether it was sharing GDB's terminal (and so
+		 assumed yes).  */
+	      fprintf_unfiltered
+		(gdb_stderr,
+		 "[tcsetpgrp failed in child_terminal_inferior: %s]\n",
+		 safe_strerror (errno));
+#endif
+	    }
 #endif
 	}
+
+      gdb_tty_state = target_terminal_state::is_inferior;
     }
-  terminal_is_ours = 0;
 }
 
 /* Put some of our terminal settings into effect,
@@ -314,7 +431,7 @@ child_terminal_inferior (struct target_ops *self)
 void
 child_terminal_ours_for_output (struct target_ops *self)
 {
-  child_terminal_ours_1 (1);
+  child_terminal_ours_1 (target_terminal_state::is_ours_for_output);
 }
 
 /* Put our terminal settings into effect.
@@ -328,35 +445,52 @@ child_terminal_ours_for_output (struct target_ops *self)
 void
 child_terminal_ours (struct target_ops *self)
 {
-  child_terminal_ours_1 (0);
+  child_terminal_ours_1 (target_terminal_state::is_ours);
 }
 
-/* output_only is not used, and should not be used unless we introduce
-   separate terminal_is_ours and terminal_is_ours_for_output
-   flags.  */
+/* Save the current terminal settings in the inferior's terminal_info
+   cache.  */
+
+void
+child_terminal_save_inferior (struct target_ops *self)
+{
+  /* Avoid attempting all the ioctl's when running in batch.  */
+  if (!gdb_has_a_terminal ())
+    return;
+
+  inferior *inf = current_inferior ();
+  terminal_info *tinfo = get_inflow_inferior_data (inf);
+
+  /* No need to save/restore if the inferior is not sharing GDB's
+     tty.  */
+  if (!sharing_input_terminal (inf))
+    return;
+
+  xfree (tinfo->ttystate);
+  tinfo->ttystate = serial_get_tty_state (stdin_serial);
+
+#ifdef HAVE_TERMIOS_H
+  tinfo->process_group = tcgetpgrp (0);
+#endif
+
+#ifdef F_GETFL
+  tinfo->tflags = fcntl (0, F_GETFL, 0);
+#endif
+}
+
+/* Switch terminal state to DESIRED_STATE, either is_ours, or
+   is_ours_for_output.  */
 
 static void
-child_terminal_ours_1 (int output_only)
+child_terminal_ours_1 (target_terminal_state desired_state)
 {
-  struct inferior *inf;
-  struct terminal_info *tinfo;
+  gdb_assert (desired_state != target_terminal_state::is_inferior);
 
-  if (terminal_is_ours)
+  /* Avoid attempting all the ioctl's when running in batch.  */
+  if (!gdb_has_a_terminal ())
     return;
 
-  terminal_is_ours = 1;
-
-  /* Checking inferior->run_terminal is necessary so that
-     if GDB is running in the background, it won't block trying
-     to do the ioctl()'s below.  Checking gdb_has_a_terminal
-     avoids attempting all the ioctl's when running in batch.  */
-
-  inf = current_inferior ();
-  tinfo = get_inflow_inferior_data (inf);
-
-  if (tinfo->run_terminal != NULL || gdb_has_a_terminal () == 0)
-    return;
-  else
+  if (gdb_tty_state != desired_state)
     {
       int result ATTRIBUTE_UNUSED;
 
@@ -364,21 +498,13 @@ child_terminal_ours_1 (int output_only)
 	 terminal's pgrp.  */
       scoped_ignore_sigttou ignore_sigttou;
 
-      xfree (tinfo->ttystate);
-      tinfo->ttystate = serial_get_tty_state (stdin_serial);
-
-#ifdef HAVE_TERMIOS_H
-      if (!inf->attach_flag)
-	/* If tcsetpgrp failed in terminal_inferior, this would give us
-	   our process group instead of the inferior's.  See
-	   terminal_inferior for details.  */
-	tinfo->process_group = tcgetpgrp (0);
-#endif
-
       /* Set tty state to our_ttystate.  */
       serial_set_tty_state (stdin_serial, our_terminal_info.ttystate);
 
-      if (job_control)
+      /* If we only want output, then leave the inferior's pgrp in the
+	 foreground, so that Ctrl-C/Ctrl-Z reach the inferior
+	 directly.  */
+      if (job_control && desired_state == target_terminal_state::is_ours)
 	{
 #ifdef HAVE_TERMIOS_H
 	  result = tcsetpgrp (0, our_terminal_info.process_group);
@@ -395,7 +521,7 @@ child_terminal_ours_1 (int output_only)
 #endif /* termios */
 	}
 
-      if (!job_control)
+      if (!job_control && desired_state == target_terminal_state::is_ours)
 	{
 	  signal (SIGINT, sigint_ours);
 #ifdef SIGQUIT
@@ -404,12 +530,98 @@ child_terminal_ours_1 (int output_only)
 	}
 
 #ifdef F_GETFL
-      tinfo->tflags = fcntl (0, F_GETFL, 0);
       result = fcntl (0, F_SETFL, our_terminal_info.tflags);
 #endif
+
+      gdb_tty_state = desired_state;
     }
 }
 
+/* Interrupt the inferior.  Implementation of target_interrupt for
+   child/native targets.  */
+
+void
+child_interrupt (struct target_ops *self)
+{
+  /* Interrupt the first inferior that has a resumed thread.  */
+  thread_info *thr;
+  thread_info *resumed = NULL;
+  ALL_NON_EXITED_THREADS (thr)
+    {
+      if (thr->executing)
+	{
+	  resumed = thr;
+	  break;
+	}
+      if (thr->suspend.waitstatus_pending_p)
+	resumed = thr;
+    }
+
+  if (resumed != NULL)
+    {
+      /* Note that unlike pressing Ctrl-C on the controlling terminal,
+	 here we only interrupt one process, not the whole process
+	 group.  This is because interrupting a process group (with
+	 either Ctrl-C or with kill(3) with negative PID) sends a
+	 SIGINT to each process in the process group, and we may not
+	 be debugging all processes in the process group.  */
+#ifndef _WIN32
+      kill (resumed->inf->pid, SIGINT);
+#endif
+    }
+}
+
+/* Pass a Ctrl-C to the inferior as-if a Ctrl-C was pressed while the
+   inferior was in the foreground.  Implementation of
+   target_pass_ctrlc for child/native targets.  */
+
+void
+child_pass_ctrlc (struct target_ops *self)
+{
+  gdb_assert (!target_terminal::is_ours ());
+
+#ifdef HAVE_TERMIOS_H
+  if (job_control)
+    {
+      pid_t term_pgrp = tcgetpgrp (0);
+
+      /* If there's any inferior sharing our terminal, pass the SIGINT
+	 to the terminal's foreground process group.  This acts just
+	 like the user typed a ^C on the terminal while the inferior
+	 was in the foreground.  Note that using a negative process
+	 number in kill() is a System V-ism.  The proper BSD interface
+	 is killpg().  However, all modern BSDs support the System V
+	 interface too.  */
+
+      if (term_pgrp != -1 && term_pgrp != our_terminal_info.process_group)
+	{
+	  kill (-term_pgrp, SIGINT);
+	  return;
+	}
+    }
+#endif
+
+  /* Otherwise, pass the Ctrl-C to the first inferior that was resumed
+     in the foreground.  */
+  inferior *inf;
+  ALL_INFERIORS (inf)
+    {
+      if (inf->terminal_state != target_terminal_state::is_ours)
+	{
+	  gdb_assert (inf->pid != 0);
+
+#ifndef _WIN32
+	  kill (inf->pid, SIGINT);
+#endif
+	  return;
+	}
+    }
+
+  /* If no inferior was resumed in the foreground, then how did the
+     !is_ours assert above pass?  */
+  gdb_assert_not_reached ("no inferior resumed in the fg found");
+}
+
 /* Per-inferior data key.  */
 static const struct inferior_data *inflow_inferior_data;
 
@@ -452,6 +664,8 @@ inflow_inferior_exit (struct inferior *inf)
 {
   struct terminal_info *info;
 
+  inf->terminal_state = target_terminal_state::is_ours;
+
   info = (struct terminal_info *) inferior_data (inf, inflow_inferior_data);
   if (info != NULL)
     {
@@ -482,6 +696,8 @@ copy_terminal_info (struct inferior *to, struct inferior *from)
   if (tinfo_from->ttystate)
     tinfo_to->ttystate
       = serial_copy_tty_state (stdin_serial, tinfo_from->ttystate);
+
+  to->terminal_state = from->terminal_state;
 }
 
 void
@@ -502,7 +718,7 @@ child_terminal_info (struct target_ops *self, const char *args, int from_tty)
       return;
     }
 
-  if (ptid_equal (inferior_ptid, null_ptid))
+  if (inferior_ptid == null_ptid)
     return;
 
   inf = current_inferior ();
@@ -692,7 +908,7 @@ static void
 pass_signal (int signo)
 {
 #ifndef _WIN32
-  kill (ptid_get_pid (inferior_ptid), SIGINT);
+  kill (inferior_ptid.pid (), SIGINT);
 #endif
 }
 
@@ -770,12 +986,10 @@ _initialize_inflow (void)
   add_info ("terminal", info_terminal_command,
 	    _("Print inferior's saved terminal status."));
 
-  terminal_is_ours = 1;
-
   /* OK, figure out whether we have job control.  */
   have_job_control ();
 
-  observer_attach_inferior_exit (inflow_inferior_exit);
+  gdb::observers::inferior_exit.attach (inflow_inferior_exit);
 
   inflow_inferior_data
     = register_inferior_data_with_cleanup (NULL, inflow_inferior_data_cleanup);
