@@ -99,7 +99,7 @@ int yyparse (void);
 
 static int yylex (void);
 
-void yyerror (const char *);
+static void yyerror (const char *);
 
 static int parse_number (int);
 
@@ -109,7 +109,7 @@ static int number_sign = 1;
 
 
 /* Line 189 of yacc.c  */
-#line 113 "m2-exp.c"
+#line 113 "m2-exp.c.tmp"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -261,7 +261,7 @@ typedef union YYSTYPE
 
 
 /* Line 214 of yacc.c  */
-#line 265 "m2-exp.c"
+#line 265 "m2-exp.c.tmp"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -273,7 +273,7 @@ typedef union YYSTYPE
 
 
 /* Line 264 of yacc.c  */
-#line 277 "m2-exp.c"
+#line 277 "m2-exp.c.tmp"
 
 #ifdef short
 # undef short
@@ -604,7 +604,7 @@ static const yytype_uint16 yyrline[] =
      370,   374,   378,   382,   386,   390,   394,   398,   400,   404,
      408,   412,   416,   420,   424,   428,   435,   441,   447,   455,
      466,   476,   485,   488,   497,   504,   508,   518,   530,   538,
-     542,   565,   608
+     542,   560,   598
 };
 #endif
 
@@ -2344,12 +2344,7 @@ yyreduce:
 			    error (_("No symbol \"%s\" in specified context."),
 				   copy_name ((yyvsp[(3) - (3)].sval)));
 			  if (symbol_read_needs_frame (sym.symbol))
-			    {
-			      if (innermost_block == 0
-				  || contained_in (sym.block,
-						   innermost_block))
-				innermost_block = sym.block;
-			    }
+			    innermost_block.update (sym);
 
 			  write_exp_elt_opcode (pstate, OP_VAR_VALUE);
 			  write_exp_elt_block (pstate, sym.block);
@@ -2360,7 +2355,7 @@ yyreduce:
   case 81:
 
 /* Line 1455 of yacc.c  */
-#line 566 "m2-exp.y"
+#line 561 "m2-exp.y"
     { struct block_symbol sym;
 			  struct field_of_this_result is_a_field_of_this;
 
@@ -2372,12 +2367,7 @@ yyreduce:
 			  if (sym.symbol)
 			    {
 			      if (symbol_read_needs_frame (sym.symbol))
-				{
-				  if (innermost_block == 0 ||
-				      contained_in (sym.block,
-						    innermost_block))
-				    innermost_block = sym.block;
-				}
+				innermost_block.update (sym);
 
 			      write_exp_elt_opcode (pstate, OP_VAR_VALUE);
 			      write_exp_elt_block (pstate, sym.block);
@@ -2405,7 +2395,7 @@ yyreduce:
   case 82:
 
 /* Line 1455 of yacc.c  */
-#line 609 "m2-exp.y"
+#line 599 "m2-exp.y"
     { (yyval.tval) = lookup_typename (parse_language (pstate),
 						parse_gdbarch (pstate),
 						copy_name ((yyvsp[(1) - (1)].sval)),
@@ -2415,7 +2405,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2420 "m2-exp.c"
+#line 2410 "m2-exp.c.tmp"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2627,7 +2617,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 616 "m2-exp.y"
+#line 606 "m2-exp.y"
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -3063,12 +3053,12 @@ m2_parse (struct parser_state *par_state)
   return yyparse ();
 }
 
-void
+static void
 yyerror (const char *msg)
 {
   if (prev_lexptr)
     lexptr = prev_lexptr;
 
-  error (_("A %s in expression, near `%s'."), (msg ? msg : "error"), lexptr);
+  error (_("A %s in expression, near `%s'."), msg, lexptr);
 }
 
